@@ -17,8 +17,8 @@ import collections
 import json
 import optparse
 import sys
+from unittest import mock
 
-import mock
 import testtools
 
 from troveclient.compat import common
