@@ -18,8 +18,6 @@
 Command-line interface to the OpenStack Trove API.
 """
 
-from __future__ import print_function
-
 import argparse
 import getpass
 import glob
@@ -30,25 +28,19 @@ import os
 import pkgutil
 import sys
 
+from keystoneauth1.identity.generic import password
+from keystoneauth1.identity.generic import token
+from keystoneauth1 import loading
+from oslo_utils import encodeutils
+from oslo_utils import importutils
 import pkg_resources
 import six
 
-from keystoneclient.auth.identity.generic import password
-from keystoneclient.auth.identity.generic import token
-from keystoneclient.auth.identity import v3 as identity
-from keystoneclient import session as ks_session
-
-from oslo_utils import encodeutils
-from oslo_utils import importutils
-
-import troveclient
+from troveclient.apiclient import exceptions as exc
 import troveclient.auth_plugin
-
 from troveclient import client
 import troveclient.extension
 from troveclient.i18n import _  # noqa
-from troveclient.openstack.common.apiclient import exceptions as exc
-
 from troveclient import utils
 from troveclient.v1 import shell as shell_v1
 
@@ -57,7 +49,7 @@ DEFAULT_OS_DATABASE_API_VERSION = "1.0"
 DEFAULT_TROVE_ENDPOINT_TYPE = 'publicURL'
 DEFAULT_TROVE_SERVICE_TYPE = 'database'
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 osprofiler_profiler = importutils.try_import("osprofiler.profiler")
 
 
@@ -66,6 +58,12 @@ class TroveClientArgumentParser(argparse.ArgumentParser):
     def __init__(self, *args, **kwargs):
         super(TroveClientArgumentParser, self).__init__(*args, **kwargs)
 
+    def add_argument(self, *args, **kwargs):
+        if kwargs.get('help') is None:
+            raise Exception(_("An argument '%s' was specified without help.")
+                            % args[0])
+        super(TroveClientArgumentParser, self).add_argument(*args, **kwargs)
+
     def error(self, message):
         """error(message: string)
 
@@ -85,12 +83,12 @@ class TroveClientArgumentParser(argparse.ArgumentParser):
 
 class OpenStackTroveShell(object):
 
-    def get_base_parser(self):
+    def get_base_parser(self, argv):
         parser = TroveClientArgumentParser(
             prog='trove',
             description=__doc__.strip(),
-            epilog='See "trove help COMMAND" '
-                   'for help on a specific command.',
+            epilog=_('See "trove help COMMAND" '
+                     'for help on a specific command.'),
             add_help=False,
             formatter_class=OpenStackHelpFormatter,
         )
@@ -103,18 +101,19 @@ class OpenStackTroveShell(object):
         parser.add_argument('--version',
                             action='version',
                             version=troveclient.__version__,
-                            help="Show program's version number and exit.")
+                            help=_("Show program's version number and exit."))
 
         parser.add_argument('--debug',
                             action='store_true',
                             default=utils.env('TROVECLIENT_DEBUG',
                                               default=False),
-                            help="Print debugging output.")
+                            help=_("Print debugging output."))
 
         parser.add_argument('--os-auth-system',
                             metavar='<auth-system>',
                             default=utils.env('OS_AUTH_SYSTEM'),
-                            help='Defaults to env[OS_AUTH_SYSTEM].')
+                            help=argparse.SUPPRESS)
+
         parser.add_argument('--os_auth_system',
                             help=argparse.SUPPRESS)
 
@@ -122,29 +121,29 @@ class OpenStackTroveShell(object):
                             metavar='<service-type>',
                             default=utils.env('OS_SERVICE_TYPE',
                                               'TROVE_SERVICE_TYPE'),
-                            help='Defaults to database for most actions.')
+                            help=_('Defaults to database for most actions.'))
         parser.add_argument('--service_type',
                             help=argparse.SUPPRESS)
 
         parser.add_argument('--service-name',
                             metavar='<service-name>',
                             default=utils.env('TROVE_SERVICE_NAME'),
-                            help='Defaults to env[TROVE_SERVICE_NAME].')
+                            help=_('Defaults to env[TROVE_SERVICE_NAME].'))
         parser.add_argument('--service_name',
                             help=argparse.SUPPRESS)
 
         parser.add_argument('--bypass-url',
                             metavar='<bypass-url>',
                             default=utils.env('TROVE_BYPASS_URL'),
-                            help='Defaults to env[TROVE_BYPASS_URL].')
+                            help=_('Defaults to env[TROVE_BYPASS_URL].'))
         parser.add_argument('--bypass_url',
                             help=argparse.SUPPRESS)
 
         parser.add_argument('--database-service-name',
                             metavar='<database-service-name>',
                             default=utils.env('TROVE_DATABASE_SERVICE_NAME'),
-                            help='Defaults to env'
-                            '[TROVE_DATABASE_SERVICE_NAME].')
+                            help=_('Defaults to env'
+                                   '[TROVE_DATABASE_SERVICE_NAME].'))
         parser.add_argument('--database_service_name',
                             help=argparse.SUPPRESS)
 
@@ -156,9 +155,12 @@ class OpenStackTroveShell(object):
                             default=utils.env(
                                 'TROVE_ENDPOINT_TYPE',
                                 default=default_trove_endpoint_type),
-                            help='Defaults to env[TROVE_ENDPOINT_TYPE] or '
-                            + 'env[OS_ENDPOINT_TYPE] or '
-                            + DEFAULT_TROVE_ENDPOINT_TYPE + '.')
+                            help=(_('Defaults to env[TROVE_ENDPOINT_TYPE] or '
+                                    'env[OS_ENDPOINT_TYPE] or '
+                                    '%(DEFAULT_TROVE_ENDPOINT_TYPE)s.') %
+                                  {'DEFAULT_TROVE_ENDPOINT_TYPE':
+                                   DEFAULT_TROVE_ENDPOINT_TYPE}))
+
         parser.add_argument('--endpoint_type',
                             help=argparse.SUPPRESS)
 
@@ -167,8 +169,8 @@ class OpenStackTroveShell(object):
                             default=utils.env(
                                 'OS_DATABASE_API_VERSION',
                                 default=DEFAULT_OS_DATABASE_API_VERSION),
-                            help='Accepts 1, defaults '
-                                 'to env[OS_DATABASE_API_VERSION].')
+                            help=_('Accepts 1, defaults '
+                                   'to env[OS_DATABASE_API_VERSION].'))
         parser.add_argument('--os_database_api_version',
                             help=argparse.SUPPRESS)
 
@@ -176,78 +178,81 @@ class OpenStackTroveShell(object):
                             metavar='<retries>',
                             type=int,
                             default=0,
-                            help='Number of retries.')
+                            help=_('Number of retries.'))
 
         parser.add_argument('--json', '--os-json-output',
                             dest='json',
                             action='store_true',
                             default=utils.env('OS_JSON_OUTPUT',
                                               default=False),
-                            help='Output JSON instead of prettyprint. '
-                                 'Defaults to env[OS_JSON_OUTPUT].')
+                            help=_('Output JSON instead of prettyprint. '
+                                   'Defaults to env[OS_JSON_OUTPUT].'))
 
         if osprofiler_profiler:
             parser.add_argument('--profile',
                                 metavar='HMAC_KEY',
                                 default=utils.env('OS_PROFILE_HMACKEY',
                                                   default=None),
-                                help='HMAC key used to encrypt context '
-                                'data when profiling the performance of an '
-                                'operation.  This key should be set to one of '
-                                'the HMAC keys configured in Trove (they are '
-                                'found in api-paste.ini, typically in '
-                                '/etc/trove).  Without the key, profiling '
-                                'will not be triggered even if it is enabled '
-                                'on the server side. '
-                                'Defaults to env[OS_PROFILE_HMACKEY].')
+                                help=_('HMAC key used to encrypt context '
+                                       'data when profiling the performance '
+                                       'of an operation. This key should be '
+                                       'set to one of the HMAC keys '
+                                       'configured in Trove (they are found '
+                                       'in configuration files, typically in '
+                                       '/etc/trove). Without the key, '
+                                       'profiling will not be triggered even '
+                                       'if it is enabled on the server side. '
+                                       'Defaults to env[OS_PROFILE_HMACKEY].'
+                                       ))
 
-        self._append_global_identity_args(parser)
+        self._append_global_identity_args(parser, argv)
 
         # The auth-system-plugins might require some extra options
         troveclient.auth_plugin.load_auth_system_opts(parser)
 
         return parser
 
-    def _append_global_identity_args(self, parser):
+    def _append_global_identity_args(self, parser, argv):
         # Register CLI identity related arguments
 
-        # Use Keystoneclient API to register common V3 CLI arguments
-        ks_session.Session.register_cli_options(parser)
-        identity.Password.register_argparse_arguments(parser)
+        # Use keystoneauth to register common  CLI arguments
+        loading.register_session_argparse_arguments(parser)
+        default_auth_plugin = 'password'
+        if 'os-token' in argv:
+            default_auth_plugin = 'token'
+        loading.register_auth_argparse_arguments(parser, argv,
+                                                 default=default_auth_plugin)
 
         parser.set_defaults(insecure=utils.env('TROVECLIENT_INSECURE',
                             default=False))
 
-        parser.add_argument('--os-tenant-name',
-                            metavar='<auth-tenant-name>',
-                            default=utils.env('OS_TENANT_NAME'),
-                            help='Tenant to request authorization on. '
-                                 'Defaults to env[OS_TENANT_NAME].')
+        parser.set_defaults(os_auth_url=utils.env('OS_AUTH_URL'))
+
+        parser.set_defaults(os_project_name=utils.env(
+            'OS_PROJECT_NAME', 'OS_TENANT_NAME'))
+        parser.set_defaults(os_project_id=utils.env(
+            'OS_PROJECT_ID', 'OS_TENANT_ID'))
+
         parser.add_argument('--os_tenant_name',
                             help=argparse.SUPPRESS)
 
-        parser.add_argument('--os-tenant-id',
-                            metavar='<tenant-id>',
-                            default=utils.env('OS_TENANT_ID'),
-                            help='Tenant to request authorization on. '
-                                 'Defaults to env[OS_TENANT_ID].')
         parser.add_argument('--os_tenant_id',
                             help=argparse.SUPPRESS)
 
         parser.add_argument('--os-auth-token',
                             default=utils.env('OS_AUTH_TOKEN'),
-                            help='Defaults to env[OS_AUTH_TOKEN]')
+                            help=argparse.SUPPRESS)
 
         parser.add_argument('--os-region-name',
                             metavar='<region-name>',
                             default=utils.env('OS_REGION_NAME'),
-                            help='Specify the region to use. '
-                                 'Defaults to env[OS_REGION_NAME].')
+                            help=_('Specify the region to use. '
+                                   'Defaults to env[OS_REGION_NAME].'))
         parser.add_argument('--os_region_name',
                             help=argparse.SUPPRESS)
 
-    def get_subcommand_parser(self, version):
-        parser = self.get_base_parser()
+    def get_subcommand_parser(self, version, argv):
+        parser = self.get_base_parser(argv)
 
         self.subcommands = {}
         subparsers = parser.add_subparsers(metavar='<subcommand>')
@@ -361,7 +366,7 @@ class OpenStackTroveShell(object):
 
     def main(self, argv):
         # Parse args once to find version and debug settings
-        parser = self.get_base_parser()
+        parser = self.get_base_parser(argv)
         (options, args) = parser.parse_known_args(argv)
         self.setup_debugging(options.debug)
         self.options = options
@@ -375,7 +380,7 @@ class OpenStackTroveShell(object):
         self._run_extension_hooks('__pre_parse_args__')
 
         subcommand_parser = self.get_subcommand_parser(
-            options.os_database_api_version)
+            options.os_database_api_version, argv)
         self.parser = subcommand_parser
 
         if options.help or not argv:
@@ -395,11 +400,22 @@ class OpenStackTroveShell(object):
 
         os_username = args.os_username
         os_password = args.os_password
-        os_tenant_name = args.os_tenant_name
+        os_project_name = getattr(args, 'os_project_name',
+                                  getattr(args, 'os_tenant_name', None))
         os_auth_url = args.os_auth_url
         os_region_name = args.os_region_name
-        os_tenant_id = args.os_tenant_id
+        os_project_id = getattr(
+            args, 'os_project_id',
+            getattr(args, 'os_tenant_id', None))
         os_auth_system = args.os_auth_system
+
+        if "v2.0" not in os_auth_url:
+            if (not args.os_project_domain_id and
+                    not args.os_project_domain_name):
+                setattr(args, "os_project_domain_id", "default")
+            if not args.os_user_domain_id and not args.os_user_domain_name:
+                setattr(args, "os_user_domain_id", "default")
+
         endpoint_type = args.endpoint_type
         insecure = args.insecure
         service_type = args.service_type
@@ -430,9 +446,9 @@ class OpenStackTroveShell(object):
 
             if not auth_plugin or not auth_plugin.opts:
                 if not os_username:
-                    raise exc.CommandError(
+                    raise exc.CommandError(_(
                         "You must provide a username "
-                        "via either --os-username or env[OS_USERNAME]")
+                        "via either --os-username or env[OS_USERNAME]"))
 
             if not os_password:
                 os_password = getpass.getpass()
@@ -442,20 +458,14 @@ class OpenStackTroveShell(object):
                     os_auth_url = auth_plugin.get_auth_url()
 
         # V3 stuff
-        project_info_provided = (self.options.os_tenant_name or
-                                 self.options.os_tenant_id or
-                                 (self.options.os_project_name and
-                                  (self.options.os_project_domain_name or
-                                   self.options.os_project_domain_id)) or
+        project_info_provided = (self.options.os_project_name or
                                  self.options.os_project_id)
 
         if (not project_info_provided):
             raise exc.CommandError(
-                _("You must provide a tenant_name, tenant_id, "
+                _("You must provide a "
                   "project_id or project_name (with "
                   "project_domain_name or project_domain_id) via "
-                  "  --os-tenant-name (env[OS_TENANT_NAME]),"
-                  "  --os-tenant-id (env[OS_TENANT_ID]),"
                   "  --os-project-id (env[OS_PROJECT_ID])"
                   "  --os-project-name (env[OS_PROJECT_NAME]),"
                   "  --os-project-domain-id "
@@ -464,47 +474,46 @@ class OpenStackTroveShell(object):
                   "(env[OS_PROJECT_DOMAIN_NAME])"))
 
         if not os_auth_url:
-            raise exc.CommandError("You must provide an auth url "
-                                   "via either --os-auth-url or "
-                                   "env[OS_AUTH_URL] or specify an "
-                                   "auth_system which defines a default "
-                                   "url with --os-auth-system or "
-                                   "env[OS_AUTH_SYSTEM]")
+            raise exc.CommandError(_("You must provide an auth url "
+                                     "via either --os-auth-url or "
+                                     "env[OS_AUTH_URL] or specify an "
+                                     "auth_system which defines a default "
+                                     "url with --os-auth-system or "
+                                     "env[OS_AUTH_SYSTEM]"))
 
         use_session = True
         if auth_plugin or bypass_url:
             use_session = False
 
-        keystone_session = None
+        ks_session = None
         keystone_auth = None
         if use_session:
-                project_id = args.os_project_id or args.os_tenant_id
-                project_name = args.os_project_name or args.os_tenant_name
+            project_id = args.os_project_id or args.os_tenant_id
+            project_name = args.os_project_name or args.os_tenant_name
 
-                keystone_session = (ks_session.Session.
-                                    load_from_cli_options(args))
-                keystone_auth = self._get_keystone_auth(
-                    keystone_session,
-                    args.os_auth_url,
-                    username=args.os_username,
-                    user_id=args.os_user_id,
-                    user_domain_id=args.os_user_domain_id,
-                    user_domain_name=args.os_user_domain_name,
-                    password=args.os_password,
-                    auth_token=args.os_auth_token,
-                    project_id=project_id,
-                    project_name=project_name,
-                    project_domain_id=args.os_project_domain_id,
-                    project_domain_name=args.os_project_domain_name)
+            ks_session = loading.load_session_from_argparse_arguments(args)
+            keystone_auth = self._get_keystone_auth(
+                ks_session,
+                args.os_auth_url,
+                username=args.os_username,
+                user_id=args.os_user_id,
+                user_domain_id=args.os_user_domain_id,
+                user_domain_name=args.os_user_domain_name,
+                password=args.os_password,
+                auth_token=args.os_auth_token,
+                project_id=project_id,
+                project_name=project_name,
+                project_domain_id=args.os_project_domain_id,
+                project_domain_name=args.os_project_domain_name)
 
         profile = osprofiler_profiler and options.profile
         if profile:
             osprofiler_profiler.init(options.profile)
 
         self.cs = client.Client(options.os_database_api_version, os_username,
-                                os_password, os_tenant_name, os_auth_url,
+                                os_password, os_project_name, os_auth_url,
                                 insecure, region_name=os_region_name,
-                                tenant_id=os_tenant_id,
+                                tenant_id=os_project_id,
                                 endpoint_type=endpoint_type,
                                 extensions=self.extensions,
                                 service_type=service_type,
@@ -516,7 +525,7 @@ class OpenStackTroveShell(object):
                                 bypass_url=bypass_url,
                                 auth_system=os_auth_system,
                                 auth_plugin=auth_plugin,
-                                session=keystone_session,
+                                session=ks_session,
                                 auth=keystone_auth)
 
         try:
@@ -526,18 +535,19 @@ class OpenStackTroveShell(object):
                 if not use_session:
                     self.cs.authenticate()
         except exc.Unauthorized:
-            raise exc.CommandError("Invalid OpenStack Trove credentials.")
+            raise exc.CommandError(_("Invalid OpenStack Trove credentials."))
         except exc.AuthorizationFailure:
-            raise exc.CommandError("Unable to authorize user")
+            raise exc.CommandError(_("Unable to authorize user"))
 
         endpoint_api_version = self.cs.get_database_api_version_from_endpoint()
 
         if endpoint_api_version != options.os_database_api_version:
-            msg = (("Database API version is set to %s "
-                    "but you are accessing a %s endpoint. "
-                    "Change its value via either --os-database-api-version "
-                    "or env[OS_DATABASE_API_VERSION]")
-                   % (options.os_database_api_version, endpoint_api_version))
+            msg = (_("Database API version is set to %(db_ver)s "
+                     "but you are accessing a %(ep_ver)s endpoint. "
+                     "Change its value via either --os-database-api-version "
+                     "or env[OS_DATABASE_API_VERSION]") %
+                   {'db_ver': options.os_database_api_version,
+                    'ep_ver': endpoint_api_version})
             # raise exc.InvalidAPIVersion(msg)
             raise exc.UnsupportedVersion(msg)
 
@@ -552,9 +562,10 @@ class OpenStackTroveShell(object):
         finally:
             if profile:
                 trace_id = osprofiler_profiler.get().get_base_id()
-                print("Trace ID: %s" % trace_id)
-                print("To display the trace, use the following command:\n"
-                      "osprofiler trace show --html %s" % trace_id)
+                print(_("Trace ID: %(trace_id)s") % {'trace_id': trace_id})
+                print(_("To display the trace, use the following command:\n"
+                        "osprofiler trace show --html %(trace_id)s")
+                      % {'trace_id': trace_id})
 
     def _run_extension_hooks(self, hook_type, *args, **kwargs):
         """Run hooks for all registered extensions."""
@@ -579,14 +590,14 @@ class OpenStackTroveShell(object):
         print(' '.join(commands | options))
 
     @utils.arg('command', metavar='<subcommand>', nargs='?',
-               help='Display help for <subcommand>.')
+               help=_('Display help for <subcommand>.'))
     def do_help(self, args):
         """Displays help about this program or one of its subcommands."""
         if args.command:
             if args.command in self.subcommands:
                 self.subcommands[args.command].print_help()
             else:
-                raise exc.CommandError("'%s' is not a valid subcommand" %
+                raise exc.CommandError(_("'%s' is not a valid subcommand") %
                                        args.command)
         else:
             self.parser.print_help()
@@ -735,14 +746,15 @@ def main():
             OpenStackTroveShell().main(map(encodeutils.safe_decode,
                                            sys.argv[1:]))
     except KeyboardInterrupt:
-        print("... terminating trove client", file=sys.stderr)
+        print(_("... terminating trove client"), file=sys.stderr)
         sys.exit(130)
     except Exception as e:
-        logger.debug(e, exc_info=1)
+        LOG.debug(e, exc_info=1)
         message = six.text_type(e)
         if not isinstance(message, six.string_types):
             message = str(message)
-        print("ERROR: %s" % encodeutils.safe_encode(message), file=sys.stderr)
+        print(_("ERROR: %s") % encodeutils.safe_encode(message),
+              file=sys.stderr)
         sys.exit(1)
 
 
