@@ -24,7 +24,9 @@ def get_version_map():
     }
 
 
-def assert_has_keys(dict, required=[], optional=[]):
+def assert_has_keys(dict, required=None, optional=None):
+    required = required or []
+    optional = optional or []
     keys = dict.keys()
     for k in required:
         try:
@@ -156,28 +158,82 @@ class FakeHTTPClient(base_client.HTTPClient):
         return r, body
 
     def get_instances(self, **kw):
+        return (
+            200,
+            {},
+            {
+                "instances": [
+                    {
+                        "id": "1234",
+                        "name": "test-member-1",
+                        "status": "ACTIVE",
+                        "addresses": [
+                            {"type": "private", "address": "10.0.0.13"}
+                        ],
+                        "volume": {"size": 2},
+                        "flavor": {"id": "02"},
+                        "region": "regionOne",
+                        "datastore": {"version": "5.6", "type": "mysql"},
+                        "tenant_id": "fake_tenant_id",
+                        "replica_of": {"id": "fake_master_id"},
+                        "access": {"is_public": False, "allowed_cidrs": []}
+                    },
+                    {
+                        "id": "5678",
+                        "name": "test-member-2",
+                        "status": "ACTIVE",
+                        "addresses": [
+                            {"type": "private", "address": "10.0.0.14"}
+                        ],
+                        "volume": {"size": 2},
+                        "flavor": {"id": "2"},
+                        "region": "regionOne",
+                        "datastore": {"version": "5.6", "type": "mysql"},
+                        "tenant_id": "fake_tenant_id",
+                        "access": {"is_public": False, "allowed_cidrs": []}
+                    },
+
+                ]
+            }
+        )
+
+    def get_instance_counts(self, **kw):
         return (200, {}, {"instances": [
             {
-                "id": "1234",
-                "name": "test-member-1",
-                "status": "ACTIVE",
-                "ip": ["10.0.0.13"],
-                "volume": {"size": 2},
-                "flavor": {"id": "02"},
-                "datastore": {"version": "5.6", "type": "mysql"}},
-            {
-                "id": "5678",
-                "name": "test-member-2",
-                "status": "ACTIVE",
-                "ip": ["10.0.0.14"],
-                "volume": {"size": 2},
-                "flavor": {"id": "2"},
-                "datastore": {"version": "5.6", "type": "mysql"}}]})
+                "module_id": "4321",
+                "module_name": "mod1",
+                "min_date": "2015-05-02T11:06:16",
+                "max_date": "2015-05-02T11:06:19",
+                "module_md5": "9db783b92a9355f70c41806659fcb77d",
+                "current": True,
+                "count": 1}]})
 
     def get_instances_1234(self, **kw):
         r = {'instance': self.get_instances()[2]['instances'][0]}
         return (200, {}, r)
 
+    def get_instance_create(self, **kw):
+        return (200, {}, {"instance": {
+                "status": "BUILD",
+                "updated": "2017-12-22T20:02:32",
+                "name": "test",
+                "created": "2017-12-22T20:02:32",
+                "networks": {
+                    "name": "test-net",
+                    "id": "net-id"
+                },
+                "id": "2468",
+                "volume": {
+                    "size": 1
+                },
+                "flavor": {
+                    "id": "310"
+                },
+                "datastore": {
+                    "version": "5.6",
+                    "type": "mysql"
+                }}})
+
     def post_instances(self, body, **kw):
         assert_has_keys(
             body['instance'],
@@ -235,10 +291,18 @@ class FakeHTTPClient(base_client.HTTPClient):
         r = {'flavor': self.get_flavors()[2]['flavors'][0]}
         return (200, {}, r)
 
+    def get_flavors_eph_rd_smaller(self, **kw):
+        r = {'flavor': self.get_flavors()[2]['flavors'][1]}
+        return (200, {}, r)
+
     def get_flavors_m1_small(self, **kw):
         r = {'flavor': self.get_flavors()[2]['flavors'][2]}
         return (200, {}, r)
 
+    def get_flavors_m1_medium(self, **kw):
+        r = {'flavor': self.get_flavors()[2]['flavors'][3]}
+        return (200, {}, r)
+
     def get_flavors_m1_uuid(self, **kw):
         r = {'flavor': self.get_flavors()[2]['flavors'][4]}
         return (200, {}, r)
@@ -251,6 +315,30 @@ class FakeHTTPClient(base_client.HTTPClient):
         r = {'flavor': self.get_flavors()[2]['flavors'][5]}
         return (200, {}, r)
 
+    def get_volume_types(self, **kw):
+        return (200, {}, {"volume_types": [
+            {
+                "id": "1",
+                "name": "vt_1",
+                "description": "Volume type #1",
+                "is_public": False},
+            {
+                "id": "10",
+                "name": "volume_type_2",
+                "description": "Volume type #2",
+                "is_public": True}]})
+
+    def get_datastores_mysql_versions_some_version_id_volume_types(self, **kw):
+        return self.get_volume_types()
+
+    def get_volume_types_1(self, **kw):
+        r = {'volume_type': self.get_volume_types()[2]['volume_types'][0]}
+        return (200, {}, r)
+
+    def get_volume_types_2(self, **kw):
+        r = {'volume_type': self.get_volume_types()[2]['volume_types'][2]}
+        return (200, {}, r)
+
     def get_clusters(self, **kw):
         return (200, {}, {"clusters": [
             {
@@ -278,8 +366,81 @@ class FakeHTTPClient(base_client.HTTPClient):
                 "id": "cls-1234"}]})
 
     def get_clusters_cls_1234(self, **kw):
-        r = {'cluster': self.get_clusters()[2]['clusters'][0]}
-        return (200, {}, r)
+        # NOTE(zhaochao): getting a cluster will load instances
+        # informations while getting the list of clusters won't,
+        # so we cannot just reuse the reponse of getting clusters
+        # for getting a cluster.
+        # The following response body is almost identical to the
+        # result of get_clusters, except the additional 'status'
+        # and 'volume' items, so all existing unitests accessing
+        # this piece of fake response will continue to work
+        # without any modification.
+        return (200, {}, {'cluster': {
+            "instances": [
+                {
+                    "type": "member",
+                    "id": "member-1",
+                    "ip": ["10.0.0.3"],
+                    "flavor": {"id": "02"},
+                    "name": "test-clstr-member-1",
+                    "status": "ACTIVE",
+                    "volume": {"size": 2}
+                },
+                {
+                    "type": "member",
+                    "id": "member-2",
+                    "ip": ["10.0.0.4"],
+                    "flavor": {"id": "2"},
+                    "name": "test-clstr-member-2",
+                    "status": "ACTIVE",
+                    "volume": {"size": 2}
+                }],
+            "updated": "2015-05-02T11:06:19",
+            "task": {"description": "No tasks for the cluster.", "id": 1,
+                     "name": "NONE"},
+            "name": "test-clstr",
+            "created": "2015-05-02T10:37:04",
+            "datastore": {"version": "7.1", "type": "vertica"},
+            "id": "cls-1234"}})
+
+    def get_cluster_instance_modules(self, **kw):
+        return (200, {}, {
+            "modules": [
+                {
+                    "auto_apply": False,
+                    "contents": None,
+                    "created": "2018-04-17 05:34:02.84",
+                    "datastore": "mariadb",
+                    "datastore_version": "all",
+                    "id": "module-1",
+                    "is_admin": False,
+                    "md5": "md5-1",
+                    "message": "Module.V1",
+                    "name": "mymod1",
+                    "removed": None,
+                    "status": "OK",
+                    "tenant": "7f1f041fc291455b83a0b3eb98140808",
+                    "type": "ping",
+                    "updated": "2018-04-17 05:34:02.84",
+                    "visible": True},
+
+                {
+                    "auto_apply": False,
+                    "contents": None,
+                    "created": "2018-04-17 05:34:02.84",
+                    "datastore": "mariadb",
+                    "datastore_version": "all",
+                    "id": "module-2",
+                    "is_admin": False,
+                    "md5": "md5-2",
+                    "message": "Module.V1",
+                    "name": "mymod2",
+                    "removed": None,
+                    "status": "OK",
+                    "tenant": "7f1f041fc291455b83a0b3eb98140808",
+                    "type": "ping",
+                    "updated": "2018-04-17 05:34:02.84",
+                    "visible": True}]})
 
     def delete_instances_1234(self, **kw):
         return (202, {}, None)
@@ -385,7 +546,13 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {}, r)
 
     def get_configurations_c_123_instances(self, **kw):
-        return (200, {}, {"instances": []})
+        return (200, {}, {"instances": [
+            {
+                "id": "1",
+                "name": "instance-1"},
+            {
+                "id": "2",
+                "name": "instance-2"}]})
 
     def delete_configurations_c_123(self, **kw):
         return (202, {}, None)
@@ -467,8 +634,13 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {}, {"modules": [{"module": {}}]})
 
     def get_modules_4321_instances(self, **kw):
+        if kw.get('count_only', False):
+            return self.get_instance_counts()
         return self.get_instances()
 
+    def put_modules_4321_instances(self, **kw):
+        return (202, {}, None)
+
     def get_instances_modules(self, **kw):
         return (200, {}, None)
 
@@ -651,3 +823,49 @@ class FakeHTTPClient(base_client.HTTPClient):
                 "to_port": 3306,
                 "security_group_id": "2",
                 "cidr": "15.0.0.0/24", "id": 3}]})
+
+    def get_quotas(self, **kw):
+        return (200, {}, {"quotas": [
+            {
+                "reserved": 1,
+                "resource": "instances",
+                "limit": 10,
+                "in_use": 2
+            },
+            {
+                "reserved": 3,
+                "resource": "backups",
+                "limit": 50,
+                "in_use": 4
+            },
+            {
+                "reserved": 5,
+                "resource": "volumes",
+                "limit": 40,
+                "in_use": 6
+            }]})
+
+    def update_instances_quota(self, **kw):
+        return (200, {}, {"quotas": {"instances": 51}})
+
+    def get_logs(self, **kw):
+        return (200, {}, {"logs": [
+            {
+                "name": "general",
+                "type": "USER",
+                "status": "Partial",
+                "published": "128",
+                "pending": "4096",
+                "container": "data_logs",
+                "prefix": "mysql-general",
+                "metafile": "mysql-general_metafile"
+            },
+            {
+                "name": "slow_query",
+                "type": "USER",
+                "status": "Ready",
+                "published": "0",
+                "pending": "128",
+                "container": "None",
+                "prefix": "None",
+                "metafile": "mysql-slow_query_metafile"}]})
