@@ -15,20 +15,22 @@
 #    under the License.
 
 from troveclient import client as trove_client
+from troveclient.v1 import backup_strategy
 from troveclient.v1 import backups
 from troveclient.v1 import clusters
 from troveclient.v1 import configurations
 from troveclient.v1 import databases
 from troveclient.v1 import datastores
-from troveclient.v1 import flavors
 from troveclient.v1 import instances
 from troveclient.v1 import limits
-# from troveclient.v1 import management
+from troveclient.v1 import management
 from troveclient.v1 import metadata
 from troveclient.v1 import modules
+from troveclient.v1 import quota
 from troveclient.v1 import root
 from troveclient.v1 import security_groups
 from troveclient.v1 import users
+from troveclient.v1 import volume_types
 
 
 class Client(object):
@@ -48,7 +50,8 @@ class Client(object):
 
     """
 
-    def __init__(self, username, password, project_id=None, auth_url='',
+    def __init__(self, username=None, password=None, project_id=None,
+                 auth_url='',
                  insecure=False, timeout=None, tenant_id=None,
                  proxy_tenant_id=None, proxy_token=None, region_name=None,
                  endpoint_type='publicURL', extensions=None,
@@ -61,10 +64,11 @@ class Client(object):
         # self.limits = limits.LimitsManager(self)
 
         # extensions
-        self.flavors = flavors.Flavors(self)
+        self.volume_types = volume_types.VolumeTypes(self)
         self.users = users.Users(self)
         self.databases = databases.Databases(self)
         self.backups = backups.Backups(self)
+        self.backup_strategies = backup_strategy.BackupStrategiesManager(self)
         self.clusters = clusters.Clusters(self)
         self.instances = instances.Instances(self)
         self.limits = limits.Limits(self)
@@ -74,22 +78,13 @@ class Client(object):
         self.datastores = datastores.Datastores(self)
         self.datastore_versions = datastores.DatastoreVersions(self)
         self.configurations = configurations.Configurations(self)
-        config_parameters = configurations.ConfigurationParameters(self)
-        self.configuration_parameters = config_parameters
+        self.configuration_parameters = configurations.ConfigurationParameters(
+            self)
         self.metadata = metadata.Metadata(self)
         self.modules = modules.Modules(self)
-
-        # self.hosts = Hosts(self)
-        # self.quota = Quotas(self)
-        # self.storage = StorageInfo(self)
-        # self.management = Management(self)
-        # self.management = MgmtClusters(self)
-        # self.mgmt_flavor = MgmtFlavors(self)
-        # self.accounts = Accounts(self)
-        # self.diagnostics = DiagnosticsInterrogator(self)
-        # self.hwinfo = HwInfoInterrogator(self)
-        # self.mgmt_config_params =
-        #       management.MgmtConfigurationParameters(self)
+        self.quota = quota.Quotas(self)
+        self.mgmt_instances = management.Management(self)
+        self.mgmt_ds_versions = management.MgmtDatastoreVersions(self)
 
         # Add in any extensions...
         if extensions:
