@@ -14,8 +14,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from __future__ import print_function
-
 import argparse
 import sys
 import time
@@ -189,11 +187,6 @@ def _find_cluster(cs, cluster):
     return utils.find_resource(cs.clusters, cluster)
 
 
-def _find_flavor(cs, flavor):
-    """Get a flavor by ID."""
-    return utils.find_resource(cs.flavors, flavor)
-
-
 def _find_volume_type(cs, volume_type):
     """Get a volume type by ID."""
     return utils.find_resource(cs.volume_types, volume_type)
@@ -224,46 +217,6 @@ def _find_configuration(cs, configuration):
     return utils.find_resource(cs.configurations, configuration)
 
 
-# Flavor related calls
-@utils.arg('--datastore_type', metavar='<datastore_type>',
-           default=None,
-           help=_('Type of the datastore. For eg: mysql.'))
-@utils.arg("--datastore_version_id", metavar="<datastore_version_id>",
-           default=None, help=_("ID of the datastore version."))
-@utils.service_type('database')
-def do_flavor_list(cs, args):
-    """Lists available flavors."""
-    if args.datastore_type and args.datastore_version_id:
-        flavors = cs.flavors.list_datastore_version_associated_flavors(
-            args.datastore_type, args.datastore_version_id)
-    elif not args.datastore_type and not args.datastore_version_id:
-        flavors = cs.flavors.list()
-    else:
-        raise exceptions.MissingArgs(['datastore_type',
-                                      'datastore_version_id'])
-
-    # Fallback to str_id where necessary.
-    _flavors = []
-    for f in flavors:
-        if not f.id and hasattr(f, 'str_id'):
-            f.id = f.str_id
-        _flavors.append(f)
-
-    utils.print_list(_flavors, ['id', 'name', 'ram', 'vcpus', 'disk',
-                                'ephemeral'],
-                     labels={'ram': 'RAM', 'vcpus': 'vCPUs', 'disk': 'Disk'},
-                     order_by='ram')
-
-
-@utils.arg('flavor', metavar='<flavor>', type=str,
-           help=_('ID or name of the flavor.'))
-@utils.service_type('database')
-def do_flavor_show(cs, args):
-    """Shows details of a flavor."""
-    flavor = _find_flavor(cs, args.flavor)
-    _print_object(flavor)
-
-
 # Volume type related calls
 @utils.arg('--datastore_type', metavar='<datastore_type>',
            default=None,
@@ -555,7 +508,7 @@ def do_update(cs, args):
 @utils.arg('flavor',
            metavar='<flavor>',
            type=str,
-           help=_('A flavor name or ID.'))
+           help=_('A flavor ID.'))
 @utils.arg('--databases', metavar='<database>',
            help=_('Optional list of databases.'),
            nargs="+", default=[])
@@ -623,7 +576,7 @@ def do_update(cs, args):
 @utils.service_type('database')
 def do_create(cs, args):
     """Creates a new instance."""
-    flavor_id = _find_flavor(cs, args.flavor).id
+    flavor_id = args.flavor
     volume = None
     if args.size is not None and args.size <= 0:
         raise exceptions.ValidationError(
@@ -686,8 +639,7 @@ def _validate_nic_info(nic_info, nic_str):
 
 
 def _get_flavor(cs, opts_str):
-    flavor_name, opts_str = _strip_option(opts_str, 'flavor', True)
-    flavor_id = _find_flavor(cs, flavor_name).id
+    flavor_id, opts_str = _strip_option(opts_str, 'flavor', True)
     return str(flavor_id), opts_str
 
 
@@ -929,12 +881,12 @@ def do_cluster_create(cs, args):
 @utils.arg('flavor',
            metavar='<flavor>',
            type=str,
-           help=_('New flavor of the instance.'))
+           help=_('New flavor ID for the instance.'))
 @utils.service_type('database')
 def do_resize_instance(cs, args):
     """Resizes an instance with a new flavor."""
     instance = _find_instance(cs, args.instance)
-    flavor_id = _find_flavor(cs, args.flavor).id
+    flavor_id = args.flavor
     cs.instances.resize_instance(instance, flavor_id)
 
 
@@ -2214,7 +2166,8 @@ def do_log_enable(cs, args):
     """Instructs Trove guest to start collecting log details."""
     try:
         instance = _find_instance(cs, args.instance)
-        log_info = cs.instances.log_enable(instance, args.log_name)
+        log_info = cs.instances.log_action(instance, args.log_name,
+                                           enable=True)
         _print_object(log_info)
     except exceptions.GuestLogNotFoundError:
         print(NO_LOG_FOUND_ERROR % {'log_name': args.log_name,
@@ -2227,15 +2180,15 @@ def do_log_enable(cs, args):
 @utils.arg('instance', metavar='<instance>',
            help=_('Id or Name of the instance.'))
 @utils.arg('log_name', metavar='<log_name>', help=_('Name of log to publish.'))
-@utils.arg('--discard', action='store_true', default=False,
-           help=_('Discard published contents of specified log.'))
+@utils.arg('--disable', action='store_true', default=False,
+           help=_('Disable the collection of the specified log.'))
 @utils.service_type('database')
 def do_log_disable(cs, args):
     """Instructs Trove guest to stop collecting log details."""
     try:
         instance = _find_instance(cs, args.instance)
-        log_info = cs.instances.log_disable(instance, args.log_name,
-                                            discard=args.discard)
+        log_info = cs.instances.log_action(instance, args.log_name,
+                                           disable=args.discard)
         _print_object(log_info)
     except exceptions.GuestLogNotFoundError:
         print(NO_LOG_FOUND_ERROR % {'log_name': args.log_name,
@@ -2257,8 +2210,8 @@ def do_log_publish(cs, args):
     """Instructs Trove guest to publish latest log entries on instance."""
     try:
         instance = _find_instance(cs, args.instance)
-        log_info = cs.instances.log_publish(
-            instance, args.log_name, disable=args.disable,
+        log_info = cs.instances.log_action(
+            instance, args.log_name, publish=True, disable=args.disable,
             discard=args.discard)
         _print_object(log_info)
     except exceptions.GuestLogNotFoundError:
@@ -2277,7 +2230,8 @@ def do_log_discard(cs, args):
     """Instructs Trove guest to discard the container of the published log."""
     try:
         instance = _find_instance(cs, args.instance)
-        log_info = cs.instances.log_discard(instance, args.log_name)
+        log_info = cs.instances.log_action(instance, args.log_name,
+                                           discard=True)
         _print_object(log_info)
     except exceptions.GuestLogNotFoundError:
         print(NO_LOG_FOUND_ERROR % {'log_name': args.log_name,
@@ -2290,8 +2244,6 @@ def do_log_discard(cs, args):
 @utils.arg('instance', metavar='<instance>',
            help=_('Id or Name of the instance.'))
 @utils.arg('log_name', metavar='<log_name>', help=_('Name of log to publish.'))
-@utils.arg('--publish', action='store_true', default=False,
-           help=_('Publish latest entries from guest before display.'))
 @utils.arg('--lines', metavar='<lines>', default=50, type=int,
            help=_('Publish latest entries from guest before display.'))
 @utils.service_type('database')
@@ -2300,7 +2252,7 @@ def do_log_tail(cs, args):
     try:
         instance = _find_instance(cs, args.instance)
         log_gen = cs.instances.log_generator(instance, args.log_name,
-                                             args.publish, args.lines)
+                                             args.lines)
         for log_part in log_gen():
             print(log_part, end="")
     except exceptions.GuestLogNotFoundError:
@@ -2314,8 +2266,6 @@ def do_log_tail(cs, args):
 @utils.arg('instance', metavar='<instance>',
            help=_('Id or Name of the instance.'))
 @utils.arg('log_name', metavar='<log_name>', help=_('Name of log to publish.'))
-@utils.arg('--publish', action='store_true', default=False,
-           help=_('Publish latest entries from guest before display.'))
 @utils.arg('--file', metavar='<file>', default=None,
            help=_('Path of file to save log to for instance.'))
 @utils.service_type('database')
@@ -2324,7 +2274,7 @@ def do_log_save(cs, args):
     try:
         instance = _find_instance(cs, args.instance)
         filename = cs.instances.log_save(instance, args.log_name,
-                                         args.publish, args.file)
+                                         filename=args.file)
         print(_('Log "%(log_name)s" written to %(file_name)s')
               % {'log_name': args.log_name,
                  'file_name': filename})
